Index: encoder/set.h
===================================================================
--- encoder/set.h	(.../x264-upstream)	(révision 18)
+++ encoder/set.h	(.../fixes1)	(révision 18)
@@ -29,6 +29,6 @@
 void x264_pps_init( x264_pps_t *pps, int i_id, x264_param_t *param, x264_sps_t *sps );
 void x264_pps_write( bs_t *s, x264_pps_t *pps );
 void x264_sei_version_write( x264_t *h, bs_t *s );
-void x264_validate_levels( x264_t *h );
+int  x264_validate_levels( x264_t *h );
 
 #endif
Index: encoder/encoder.c
===================================================================
--- encoder/encoder.c	(.../x264-upstream)	(révision 18)
+++ encoder/encoder.c	(.../fixes1)	(révision 18)
@@ -427,10 +427,34 @@
             x264_log( h, X264_LOG_ERROR, "invalid level_idc: %d\n", h->param.i_level_idc );
             return -1;
         }
-        if( h->param.analyse.i_mv_range <= 0 )
-            h->param.analyse.i_mv_range = l->mv_range;
+        if( h->param.analyse.i_horizontal_mv_range <= 0 )
+            h->param.analyse.i_horizontal_mv_range = 2048; // This is a level independent limit
         else
-            h->param.analyse.i_mv_range = x264_clip3(h->param.analyse.i_mv_range, 32, 2048);
+            h->param.analyse.i_horizontal_mv_range = x264_clip3(h->param.analyse.i_horizontal_mv_range, 32, 2048);
+        if( h->param.analyse.i_vertical_mv_range <= 0 )
+            h->param.analyse.i_vertical_mv_range = l->mv_range;
+        else
+            h->param.analyse.i_vertical_mv_range = x264_clip3(h->param.analyse.i_vertical_mv_range, 32, l->mv_range);
+        /* Validate direct_8x8_inference_flag: */
+        if( h->param.i_direct_8x8_inference_flag < 0 )
+        {
+            /* The flag isn't forced, set it to 0 if the level doesn't
+             * limit it to 1 and it is possible to generate P macroblocks
+             * with partitions smaller than 8x8 */
+            h->param.i_direct_8x8_inference_flag = l->direct8x8 ? 1 : !(h->param.analyse.inter & X264_ANALYSE_PSUB8x8);
+        }
+        else
+        {
+            h->param.i_direct_8x8_inference_flag = x264_clip3( h->param.i_direct_8x8_inference_flag, 0, 1 );
+            /* if the level limits this flag to 1, check it isn't forced to 0 */
+            if (h->param.i_direct_8x8_inference_flag < l->direct8x8)
+            {
+                x264_log( h, X264_LOG_ERROR, "invalid direct_8x8_inference_flag %d for level %d\n", h->param.i_direct_8x8_inference_flag, h->param.i_level_idc );
+               if (!h->param.b_allow_invalid_bitstreams) {
+                   return -1;
+                }
+            }
+        }
     }
 
     if( h->param.rc.f_qblur < 0 )
@@ -457,6 +481,50 @@
 }
 
 /****************************************************************************
+ * x264_encoder_delete:
+ ****************************************************************************/
+static void x264_encoder_delete ( x264_t *h )
+{
+    int i;
+
+    /* param */
+    /* allocation is done by strdup, must use free here, not x264_free */
+    if( h->param.rc.psz_stat_out )
+        free( h->param.rc.psz_stat_out );
+    if( h->param.rc.psz_stat_in )
+        free( h->param.rc.psz_stat_in );
+    if( h->param.rc.psz_rc_eq )
+        free( h->param.rc.psz_rc_eq );
+
+    /* bitstream */
+    x264_free( h->out.p_bitstream );
+
+    /* frames */
+    for( i = 0; i < X264_BFRAME_MAX + 3; i++ )
+    {
+        if( h->frames.current[i] ) x264_frame_delete( h->frames.current[i] );
+        if( h->frames.next[i] )    x264_frame_delete( h->frames.next[i] );
+        if( h->frames.unused[i] )  x264_frame_delete( h->frames.unused[i] );
+    }
+    /* ref frames */
+    for( i = 0; i < X264_MAX_REFERENCES; i++ )
+    {
+        if( h->frames.reference[i] ) x264_frame_delete( h->frames.reference[i] );
+    }
+
+    /* cache */
+    x264_macroblock_cache_end( h );
+
+    /* rc */
+    x264_ratecontrol_delete( h );
+
+    /* thread */
+    for( i = 1; i < X264_THREAD_MAX; i++ )
+        x264_free( h->thread[i] );
+    x264_free( h );
+}
+
+/****************************************************************************
  * x264_encoder_open:
  ****************************************************************************/
 x264_t *x264_encoder_open   ( x264_param_t *param )
@@ -464,31 +532,34 @@
     x264_t *h = x264_malloc( sizeof( x264_t ) );
     int i;
 
+    if( NULL == h )
+    {
+        return NULL;
+    }
+
     memset( h, 0, sizeof( x264_t ) );
 
     /* Create a copy of param */
     memcpy( &h->param, param, sizeof( x264_param_t ) );
 
+    if( h->param.rc.psz_stat_out )
+        h->param.rc.psz_stat_out = strdup( h->param.rc.psz_stat_out );
+    if( h->param.rc.psz_stat_in )
+        h->param.rc.psz_stat_in = strdup( h->param.rc.psz_stat_in );
+    if( h->param.rc.psz_rc_eq )
+        h->param.rc.psz_rc_eq = strdup( h->param.rc.psz_rc_eq );
+
     if( x264_validate_parameters( h ) < 0 )
     {
-        x264_free( h );
-        return NULL;
+        goto fail;
     }
 
     if( h->param.psz_cqm_file )
         if( x264_cqm_parse_file( h, h->param.psz_cqm_file ) < 0 )
         {
-            x264_free( h );
-            return NULL;
+            goto fail;
         }
 
-    if( h->param.rc.psz_stat_out )
-        h->param.rc.psz_stat_out = strdup( h->param.rc.psz_stat_out );
-    if( h->param.rc.psz_stat_in )
-        h->param.rc.psz_stat_in = strdup( h->param.rc.psz_stat_in );
-    if( h->param.rc.psz_rc_eq )
-        h->param.rc.psz_rc_eq = strdup( h->param.rc.psz_rc_eq );
-
     /* VUI */
     if( h->param.vui.i_sar_width > 0 && h->param.vui.i_sar_height > 0 )
     {
@@ -524,7 +595,7 @@
     h->out.i_bitstream = X264_MAX( 1000000, h->param.i_width * h->param.i_height * 1.7
         * ( h->param.rc.b_cbr ? pow( 0.5, h->param.rc.i_qp_min )
           : pow( 0.5, h->param.rc.i_qp_constant ) * X264_MAX( 1, h->param.rc.f_ip_factor )));
-    h->out.p_bitstream = x264_malloc( h->out.i_bitstream );
+    CHECKED_MALLOC( h->out.p_bitstream, h->out.i_bitstream );
 
     h->i_frame = 0;
     h->i_frame_num = 0;
@@ -536,7 +607,10 @@
     h->pps = &h->pps_array[0];
     x264_pps_init( h->pps, h->param.i_sps_id, &h->param, h->sps);
 
-    x264_validate_levels( h );
+    if (x264_validate_levels( h ) < 0)
+    {
+        goto fail;
+    }
 
     x264_cqm_init( h );
     
@@ -550,23 +624,18 @@
     h->frames.b_have_lowres = !h->param.rc.b_stat_read
         && ( h->param.rc.b_cbr || h->param.rc.i_rf_constant || h->param.b_bframe_adaptive );
 
-    for( i = 0; i < X264_BFRAME_MAX + 3; i++ )
-    {
-        h->frames.current[i] = NULL;
-        h->frames.next[i]    = NULL;
-        h->frames.unused[i]  = NULL;
-    }
+    /* Note that all frames pointers are initialised at NULL by memset 0 upper */
     for( i = 0; i < 1 + h->frames.i_delay; i++ )
     {
         h->frames.unused[i] =  x264_frame_new( h );
         if( !h->frames.unused[i] )
-            return NULL;
+            goto fail;
     }
     for( i = 0; i < h->frames.i_max_dpb; i++ )
     {
         h->frames.reference[i] = x264_frame_new( h );
         if( !h->frames.reference[i] )
-            return NULL;
+            goto fail;
     }
     h->frames.reference[h->frames.i_max_dpb] = NULL;
     h->frames.i_last_idr = - h->param.i_keyint_max;
@@ -579,7 +648,7 @@
     h->fdec = h->frames.reference[0];
 
     if( x264_macroblock_cache_init( h ) < 0 )
-        return NULL;
+        goto fail;
     x264_rdo_init( );
 
     /* init CPU functions */
@@ -601,7 +670,7 @@
 
     /* rate control */
     if( x264_ratecontrol_new( h ) < 0 )
-        return NULL;
+        goto fail;
 
     x264_log( h, X264_LOG_INFO, "using cpu capabilities %s%s%s%s%s%s\n",
              param->cpu&X264_CPU_MMX ? "MMX " : "",
@@ -614,7 +683,7 @@
     h->thread[0] = h;
     h->i_thread_num = 0;
     for( i = 1; i < param->i_threads; i++ )
-        h->thread[i] = x264_malloc( sizeof(x264_t) );
+        CHECKED_MALLOC( h->thread[i], sizeof(x264_t) );
 
 #ifdef DEBUG_DUMP_FRAME
     {
@@ -632,6 +701,10 @@
 #endif
 
     return h;
+
+fail:
+    x264_encoder_delete( h );
+    return NULL;
 }
 
 /****************************************************************************
@@ -1788,34 +1861,6 @@
             x264_log( h, X264_LOG_INFO, "kb/s:%.1f\n", f_bitrate );
     }
 
-    /* frames */
-    for( i = 0; i < X264_BFRAME_MAX + 3; i++ )
-    {
-        if( h->frames.current[i] ) x264_frame_delete( h->frames.current[i] );
-        if( h->frames.next[i] )    x264_frame_delete( h->frames.next[i] );
-        if( h->frames.unused[i] )  x264_frame_delete( h->frames.unused[i] );
-    }
-    /* ref frames */
-    for( i = 0; i < h->frames.i_max_dpb; i++ )
-    {
-        x264_frame_delete( h->frames.reference[i] );
-    }
-
-    /* rc */
-    x264_ratecontrol_delete( h );
-
-    /* param */
-    if( h->param.rc.psz_stat_out )
-        free( h->param.rc.psz_stat_out );
-    if( h->param.rc.psz_stat_in )
-        free( h->param.rc.psz_stat_in );
-    if( h->param.rc.psz_rc_eq )
-        free( h->param.rc.psz_rc_eq );
-
-    x264_macroblock_cache_end( h );
-    x264_free( h->out.p_bitstream );
-    for( i = 1; i < h->param.i_threads; i++ )
-        x264_free( h->thread[i] );
-    x264_free( h );
+    x264_encoder_delete( h );
 }
 
Index: encoder/set.c
===================================================================
--- encoder/set.c	(.../x264-upstream)	(révision 18)
+++ encoder/set.c	(.../fixes1)	(révision 18)
@@ -130,13 +130,7 @@
     sps->i_mb_height= ( param->i_height + 15 )/ 16;
     sps->b_frame_mbs_only = 1;
     sps->b_mb_adaptive_frame_field = 0;
-    sps->b_direct8x8_inference = 0;
-    if( sps->b_frame_mbs_only == 0 ||
-        !(param->analyse.inter & X264_ANALYSE_PSUB8x8) )
-    {
-        sps->b_direct8x8_inference = 1;
-    }
-
+    sps->b_direct8x8_inference = param->i_direct_8x8_inference_flag;
     sps->crop.i_left   = 0;
     sps->crop.i_top    = 0;
     sps->crop.i_right  = (- param->i_width)  & 15;
@@ -207,8 +201,8 @@
         sps->vui.b_motion_vectors_over_pic_boundaries = 1;
         sps->vui.i_max_bytes_per_pic_denom = 0;
         sps->vui.i_max_bits_per_mb_denom = 0;
-        sps->vui.i_log2_max_mv_length_horizontal =
-        sps->vui.i_log2_max_mv_length_vertical = (int)(log(param->analyse.i_mv_range*4-1)/log(2)) + 1;
+        sps->vui.i_log2_max_mv_length_horizontal = (int)(log(param->analyse.i_horizontal_mv_range*4-1)/log(2)) + 1;
+        sps->vui.i_log2_max_mv_length_vertical = (int)(log(param->analyse.i_vertical_mv_range*4-1)/log(2)) + 1;
     }
 }
 
@@ -518,7 +512,7 @@
     { 0 }
 };
 
-void x264_validate_levels( x264_t *h )
+int x264_validate_levels( x264_t *h )
 {
     int mbs;
 
@@ -530,20 +524,29 @@
     if( l->frame_size < mbs
         || l->frame_size*8 < h->sps->i_mb_width * h->sps->i_mb_width
         || l->frame_size*8 < h->sps->i_mb_height * h->sps->i_mb_height )
-        x264_log( h, X264_LOG_WARNING, "frame MB size (%dx%d) > level limit (%d)\n",
-                  h->sps->i_mb_width, h->sps->i_mb_height, l->frame_size );
+    {
+        x264_log( h, X264_LOG_ERROR, "frame MB size (%dx%d=%d) > level limit (%d)\n",
+                  h->sps->i_mb_width, h->sps->i_mb_height, mbs, l->frame_size );
+        if (!h->param.b_allow_invalid_bitstreams)
+            return -1;
+    }
 
 #define CHECK( name, limit, val ) \
     if( (val) > (limit) ) \
-        x264_log( h, X264_LOG_WARNING, name " (%d) > level limit (%d)\n", (int)(val), (limit) );
+    { \
+        x264_log( h, X264_LOG_ERROR, name " (%d) > level limit (%d), try increase level\n", (int)(val), (limit) ); \
+        if (!h->param.b_allow_invalid_bitstreams) \
+            return -1; \
+    }
 
     CHECK( "DPB size", l->dpb, mbs * 384 * h->sps->i_num_ref_frames );
     CHECK( "VBV bitrate", l->bitrate, h->param.rc.i_vbv_max_bitrate );
     CHECK( "VBV buffer", l->cpb, h->param.rc.i_vbv_buffer_size );
-    CHECK( "MV range", l->mv_range, h->param.analyse.i_mv_range );
+    CHECK( "MV range", l->mv_range, h->param.analyse.i_vertical_mv_range );
 
     if( h->param.i_fps_den > 0 )
         CHECK( "MB rate", l->mbps, (int64_t)mbs * h->param.i_fps_num / h->param.i_fps_den );
 
     /* TODO check the rest of the limits */
+    return 0;
 }
Index: encoder/ratecontrol.c
===================================================================
--- encoder/ratecontrol.c	(.../x264-upstream)	(révision 18)
+++ encoder/ratecontrol.c	(.../fixes1)	(révision 18)
@@ -194,7 +194,8 @@
 
     x264_cpu_restore( h->param.cpu );
 
-    h->rc = rc = x264_malloc( h->param.i_threads * sizeof(x264_ratecontrol_t) );
+    CHECKED_MALLOC( rc, h->param.i_threads * sizeof(x264_ratecontrol_t) );
+    h->rc = rc;
     memset( rc, 0, h->param.i_threads * sizeof(x264_ratecontrol_t) );
 
     rc->b_abr = ( h->param.rc.b_cbr || h->param.rc.i_rf_constant ) && !h->param.rc.b_stat_read;
@@ -215,7 +216,7 @@
     if( h->param.rc.i_rf_constant && h->param.rc.b_stat_read )
     {
         x264_log(h, X264_LOG_ERROR, "constant rate-factor is incompatible with 2pass.\n");
-        return -1;
+        goto fail;
     }
     if( h->param.rc.i_vbv_buffer_size && !h->param.rc.b_cbr && !h->param.rc.i_rf_constant )
         x264_log(h, X264_LOG_WARNING, "VBV is incompatible with constant QP.\n");
@@ -294,7 +295,7 @@
     rc->pred_b_from_p = rc->pred[0];
 
     if( parse_zones( h ) < 0 )
-        return -1;
+        goto fail;
 
     /* Load stat file and init 2pass algo */
     if( h->param.rc.b_stat_read )
@@ -304,10 +305,10 @@
         /* read 1st pass stats */
         assert( h->param.rc.psz_stat_in );
         stats_buf = stats_in = x264_slurp_file( h->param.rc.psz_stat_in );
-        if( !stats_buf )
+        if( NULL == stats_buf )
         {
             x264_log(h, X264_LOG_ERROR, "ratecontrol_init: can't open stats file\n");
-            return -1;
+            goto fail;
         }
 
         /* check whether 1st pass options were compatible with current options */
@@ -317,7 +318,10 @@
             char *opts = stats_buf;
             stats_in = strchr( stats_buf, '\n' );
             if( !stats_in )
-                return -1;
+            {
+                x264_free( stats_buf );
+                goto fail;
+            }
             *stats_in = '\0';
             stats_in++;
 
@@ -326,7 +330,8 @@
             {
                 x264_log( h, X264_LOG_ERROR, "different number of B-frames than 1st pass (%d vs %d)\n",
                           h->param.i_bframe, i );
-                return -1;
+                x264_free( stats_buf );
+                goto fail;
             }
 
             /* since B-adapt doesn't (yet) take into account B-pyramid,
@@ -350,7 +355,8 @@
         if(i==0)
         {
             x264_log(h, X264_LOG_ERROR, "empty stats file\n");
-            return -1;
+            x264_free( stats_buf );
+            goto fail;
         }
         rc->num_entries = i;
 
@@ -363,13 +369,19 @@
         {
             x264_log( h, X264_LOG_ERROR, "2nd pass has more frames than 1st pass (%d vs %d)\n",
                       h->param.i_frame_total, rc->num_entries );
-            return -1;
+            x264_free( stats_buf );
+            goto fail;
         }
 
         /* FIXME: ugly padding because VfW drops delayed B-frames */
         rc->num_entries += h->param.i_bframe;
 
         rc->entry = (ratecontrol_entry_t*) x264_malloc(rc->num_entries * sizeof(ratecontrol_entry_t));
+        if( NULL == rc->entry )
+        {
+            x264_free( stats_buf );
+            goto fail;
+        }
         memset(rc->entry, 0, rc->num_entries * sizeof(ratecontrol_entry_t));
 
         /* init all to skipped p frames */
@@ -401,7 +413,8 @@
             if(frame_number < 0 || frame_number >= rc->num_entries)
             {
                 x264_log(h, X264_LOG_ERROR, "bad frame number (%d) at stats line %d\n", frame_number, i);
-                return -1;
+                x264_free( stats_buf );
+                goto fail;
             }
             rce = &rc->entry[frame_number];
             rce->direct_mode = 0;
@@ -442,7 +455,7 @@
     {
         char *p;
 
-        rc->psz_stat_file_tmpname = x264_malloc( strlen(h->param.rc.psz_stat_out) + 6 );
+        CHECKED_MALLOC( rc->psz_stat_file_tmpname, strlen(h->param.rc.psz_stat_out) + 6 );
         strcpy( rc->psz_stat_file_tmpname, h->param.rc.psz_stat_out );
         strcat( rc->psz_stat_file_tmpname, ".temp" );
 
@@ -450,28 +463,35 @@
         if( rc->p_stat_file_out == NULL )
         {
             x264_log(h, X264_LOG_ERROR, "ratecontrol_init: can't open stats file\n");
-            return -1;
+            goto fail;
         }
 
         p = x264_param2string( &h->param, 1 );
+        if( NULL == p )
+            goto fail;
         fprintf( rc->p_stat_file_out, "#options: %s\n", p );
         x264_free( p );
     }
 
     return 0;
+    
+fail:
+    x264_ratecontrol_delete( h );
+    return -1;
 }
 
 static int parse_zones( x264_t *h )
 {
     x264_ratecontrol_t *rc = h->rc;
     int i;
+    h->param.rc.zones = NULL;
     if( h->param.rc.psz_zones && !h->param.rc.i_zones )
     {
         char *p;
         h->param.rc.i_zones = 1;
         for( p = h->param.rc.psz_zones; *p; p++ )
             h->param.rc.i_zones += (*p == '/');
-        h->param.rc.zones = x264_malloc( h->param.rc.i_zones * sizeof(x264_zone_t) );
+        CHECKED_MALLOC( h->param.rc.zones, h->param.rc.i_zones * sizeof(x264_zone_t) );
         p = h->param.rc.psz_zones;
         for( i = 0; i < h->param.rc.i_zones; i++)
         {
@@ -485,7 +505,7 @@
                 char *slash = strchr(p, '/');
                 if(slash) *slash = '\0';
                 x264_log( h, X264_LOG_ERROR, "invalid zone: \"%s\"\n", p );
-                return -1;
+                goto fail;
             }
             p = strchr(p, '/') + 1;
         }
@@ -500,22 +520,25 @@
             {
                 x264_log( h, X264_LOG_ERROR, "invalid zone: start=%d end=%d\n",
                           z.i_start, z.i_end );
-                return -1;
+                goto fail;
             }
             else if( !z.b_force_qp && z.f_bitrate_factor <= 0 )
             {
                 x264_log( h, X264_LOG_ERROR, "invalid zone: bitrate_factor=%f\n",
                           z.f_bitrate_factor );
-                return -1;
+                goto fail;
             }
         }
 
         rc->i_zones = h->param.rc.i_zones;
-        rc->zones = x264_malloc( rc->i_zones * sizeof(x264_zone_t) );
-        memcpy( rc->zones, h->param.rc.zones, rc->i_zones * sizeof(x264_zone_t) );
+        rc->zones = h->param.rc.zones;
+        h->param.rc.zones = NULL;
     }
 
     return 0;
+fail:
+    x264_free( h->param.rc.zones );
+    return -1;
 }
 
 void x264_ratecontrol_summary( x264_t *h )
@@ -534,6 +557,8 @@
 {
     x264_ratecontrol_t *rc = h->rc;
 
+    if( NULL == rc ) return;
+
     if( rc->p_stat_file_out )
     {
         fclose( rc->p_stat_file_out );
@@ -543,8 +568,8 @@
                 x264_log( h, X264_LOG_ERROR, "failed to rename \"%s\" to \"%s\"\n",
                           rc->psz_stat_file_tmpname, h->param.rc.psz_stat_out );
             }
-        x264_free( rc->psz_stat_file_tmpname );
     }
+    x264_free( rc->psz_stat_file_tmpname );
     x264_free( rc->entry );
     x264_free( rc->zones );
     x264_free( rc );
Index: encoder/analyse.c
===================================================================
--- encoder/analyse.c	(.../x264-upstream)	(révision 18)
+++ encoder/analyse.c	(.../fixes1)	(révision 18)
@@ -218,26 +218,29 @@
     if( h->sh.i_type != SLICE_TYPE_I )
     {
         int i;
-        int i_fmv_range = h->param.analyse.i_mv_range - 16;
+        const int i_h_fmv_range = h->param.analyse.i_horizontal_mv_range - 16;
+        const int i_v_fmv_range = h->param.analyse.i_vertical_mv_range - 16;
 
         /* Calculate max allowed MV range */
-#define CLIP_FMV(mv) x264_clip3( mv, -i_fmv_range, i_fmv_range )
         h->mb.mv_min[0] = 4*( -16*h->mb.i_mb_x - 24 );
         h->mb.mv_max[0] = 4*( 16*( h->sps->i_mb_width - h->mb.i_mb_x - 1 ) + 24 );
-        h->mb.mv_min_fpel[0] = CLIP_FMV( -16*h->mb.i_mb_x - 8 );
-        h->mb.mv_max_fpel[0] = CLIP_FMV( 16*( h->sps->i_mb_width - h->mb.i_mb_x - 1 ) + 8 );
+#define CLIP_H_FMV(mv) x264_clip3( mv, -i_h_fmv_range, i_h_fmv_range )
+        h->mb.mv_min_fpel[0] = CLIP_H_FMV( -16*h->mb.i_mb_x - 8 );
+        h->mb.mv_max_fpel[0] = CLIP_H_FMV( 16*( h->sps->i_mb_width - h->mb.i_mb_x - 1 ) + 8 );
+#undef CLIP_H_FMV
         h->mb.mv_min_spel[0] = 4*( h->mb.mv_min_fpel[0] - 16 );
         h->mb.mv_max_spel[0] = 4*( h->mb.mv_max_fpel[0] + 16 );
         if( h->mb.i_mb_x == 0)
         {
             h->mb.mv_min[1] = 4*( -16*h->mb.i_mb_y - 24 );
             h->mb.mv_max[1] = 4*( 16*( h->sps->i_mb_height - h->mb.i_mb_y - 1 ) + 24 );
-            h->mb.mv_min_fpel[1] = CLIP_FMV( -16*h->mb.i_mb_y - 8 );
-            h->mb.mv_max_fpel[1] = CLIP_FMV( 16*( h->sps->i_mb_height - h->mb.i_mb_y - 1 ) + 8 );
+#define CLIP_V_FMV(mv) x264_clip3( mv, -i_v_fmv_range, i_v_fmv_range )
+            h->mb.mv_min_fpel[1] = CLIP_V_FMV( -16*h->mb.i_mb_y - 8 );
+            h->mb.mv_max_fpel[1] = CLIP_V_FMV( 16*( h->sps->i_mb_height - h->mb.i_mb_y - 1 ) + 8 );
+#undef CLIP_V_FMV
             h->mb.mv_min_spel[1] = 4*( h->mb.mv_min_fpel[1] - 16 );
             h->mb.mv_max_spel[1] = 4*( h->mb.mv_max_fpel[1] + 16 );
         }
-#undef CLIP_FMV
 
         a->l0.me16x16.cost =
         a->l0.i_rd16x16    =
Index: x264.c
===================================================================
--- x264.c	(.../x264-upstream)	(révision 18)
+++ x264.c	(.../fixes1)	(révision 18)
@@ -202,6 +202,10 @@
              "                                  (p4x4 requires p8x8. i8x8 requires --8x8dct.)\n"
              "      --direct <string>       Direct MV prediction mode [\"%s\"]\n"
              "                                  - none, spatial, temporal, auto\n"
+             "      --direct-8x8 <-1|0|1>   Direct prediction size [%d]\n"
+             "                                  -  0: 4x4\n"
+             "                                  -  1: 8x8\n"
+             "                                  - -1: smallest possible according to level\n"
              "  -w, --weightb               Weighted prediction for B-frames\n"
              "      --me <string>           Integer pixel motion estimation method [\"%s\"]\n"
              "                                  - dia: diamond search, radius 1 (fast)\n"
@@ -314,6 +318,7 @@
             defaults->rc.f_complexity_blur,
             defaults->rc.f_qblur,
             strtable_lookup( x264_direct_pred_names, defaults->analyse.i_direct_mv_pred ),
+            defaults->i_direct_8x8_inference_flag,
             strtable_lookup( x264_motion_est_names, defaults->analyse.i_me_method ),
             defaults->analyse.i_me_range,
             defaults->analyse.i_subpel_refine,
@@ -452,6 +457,7 @@
 #define OPT_THREAD_INPUT 320
 #define OPT_NO_DCT_DECIMATE 321
 #define OPT_SPS_ID 322
+#define OPT_DIRECT_8X8 323
 
         static struct option long_options[] =
         {
@@ -481,6 +487,7 @@
             { "output",  required_argument, NULL, 'o' },
             { "analyse", required_argument, NULL, 'A' },
             { "direct",  required_argument, NULL, OPT_DIRECT },
+            { "direct-8x8",  required_argument, NULL, OPT_DIRECT_8X8 },
             { "weightb", no_argument,       NULL, 'w' },
             { "me",      required_argument, NULL, OPT_ME },
             { "merange", required_argument, NULL, OPT_MERANGE },
@@ -703,6 +710,9 @@
             case OPT_DIRECT:
                 b_error |= parse_enum( optarg, x264_direct_pred_names, &param->analyse.i_direct_mv_pred );
                 break;
+            case OPT_DIRECT_8X8:
+                param->i_direct_8x8_inference_flag = atoi(optarg);
+                break;
             case 'w':
                 param->analyse.b_weighted_bipred = 1;
                 break;
Index: common/macroblock.c
===================================================================
--- common/macroblock.c	(.../x264-upstream)	(révision 18)
+++ common/macroblock.c	(.../fixes1)	(révision 18)
@@ -863,23 +863,21 @@
     memset( h->mb.cache.ref[1], -2, X264_SCAN8_SIZE * sizeof( int8_t ) );
 
     return 0;
-fail: return -1;
+fail:
+    x264_macroblock_cache_end( h );
+    return -1;
 }
 void x264_macroblock_cache_end( x264_t *h )
 {
     int i, j;
     for( i=0; i<2; i++ )
     {
-        int i_refs = i ? 1 + h->param.b_bframe_pyramid : h->param.i_frame_reference;
-        for( j=0; j < i_refs; j++ )
+        for( j=0; j < 16; j++ )
             x264_free( h->mb.mvr[i][j] );
     }
-    if( h->param.b_cabac )
-    {
-        x264_free( h->mb.chroma_pred_mode );
-        x264_free( h->mb.mvd[0] );
-        x264_free( h->mb.mvd[1] );
-    }
+    x264_free( h->mb.chroma_pred_mode );
+    x264_free( h->mb.mvd[0] );
+    x264_free( h->mb.mvd[1] );
     x264_free( h->mb.intra4x4_pred_mode );
     x264_free( h->mb.non_zero_count );
     x264_free( h->mb.mb_transform_size );
Index: common/common.c
===================================================================
--- common/common.c	(.../x264-upstream)	(révision 18)
+++ common/common.c	(.../fixes1)	(révision 18)
@@ -72,6 +72,7 @@
     param->b_bframe_adaptive = 1;
     param->i_bframe_bias = 0;
     param->b_bframe_pyramid = 0;
+    param->i_direct_8x8_inference_flag = -1;
 
     param->b_deblocking_filter = 1;
     param->i_deblocking_filter_alphac0 = 0;
@@ -118,7 +119,8 @@
     param->analyse.i_me_range = 16;
     param->analyse.i_subpel_refine = 5;
     param->analyse.b_chroma_me = 1;
-    param->analyse.i_mv_range = -1; // set from level_idc
+    param->analyse.i_horizontal_mv_range = -1;
+    param->analyse.i_vertical_mv_range = -1; // set from level_idc
     param->analyse.i_chroma_qp_offset = 0;
     param->analyse.b_fast_pskip = 1;
     param->analyse.b_dct_decimate = 1;
@@ -132,6 +134,7 @@
     memset( param->cqm_8iy, 16, 64 );
     memset( param->cqm_8py, 16, 64 );
 
+    param->b_allow_invalid_bitstreams = 0;
     param->b_repeat_headers = 1;
     param->b_aud = 0;
 }
@@ -424,7 +427,7 @@
 {
     int b_error = 0;
     int i_size;
-    char *buf;
+    char *buf = NULL;
     FILE *fh = fopen( filename, "rb" );
     if( !fh )
         return NULL;
@@ -432,21 +435,23 @@
     b_error |= ( i_size = ftell( fh ) ) <= 0;
     b_error |= fseek( fh, 0, SEEK_SET ) < 0;
     if( b_error )
-        return NULL;
+        goto fail;
     buf = x264_malloc( i_size+2 );
-    if( buf == NULL )
-        return NULL;
+    if( NULL == buf )
+        goto fail;
     b_error |= fread( buf, 1, i_size, fh ) != i_size;
     if( buf[i_size-1] != '\n' )
         buf[i_size++] = '\n';
     buf[i_size] = 0;
     fclose( fh );
-    if( b_error )
+    if( !b_error )
     {
-        x264_free( buf );
-        return NULL;
+        return buf;
     }
-    return buf;
+fail:
+    x264_free( buf );
+    fclose( fh );
+    return NULL;
 }
 
 /****************************************************************************
@@ -457,6 +462,8 @@
     char *buf = x264_malloc( 1000 );
     char *s = buf;
 
+    if ( NULL == buf ) return NULL;
+
     if( b_res )
     {
         s += sprintf( s, "%dx%d ", p->i_width, p->i_height );
@@ -485,10 +492,10 @@
     s += sprintf( s, " bframes=%d", p->i_bframe );
     if( p->i_bframe )
     {
-        s += sprintf( s, " b_pyramid=%d b_adapt=%d b_bias=%d direct=%d wpredb=%d bime=%d",
+        s += sprintf( s, " b_pyramid=%d b_adapt=%d b_bias=%d direct=%d d8x8=%d wpredb=%d bime=%d",
                       p->b_bframe_pyramid, p->b_bframe_adaptive, p->i_bframe_bias,
-                      p->analyse.i_direct_mv_pred, p->analyse.b_weighted_bipred,
-                      p->analyse.b_bidir_me );
+                      p->analyse.i_direct_mv_pred, p->i_direct_8x8_inference_flag,
+                      p->analyse.b_weighted_bipred, p->analyse.b_bidir_me );
     }
 
     s += sprintf( s, " keyint=%d keyint_min=%d scenecut=%d",
Index: common/set.c
===================================================================
--- common/set.c	(.../x264-upstream)	(révision 18)
+++ common/set.c	(.../fixes1)	(révision 18)
@@ -170,7 +170,7 @@
     h->param.i_cqm_preset = X264_CQM_CUSTOM;
     
     buf = x264_slurp_file( filename );
-    if( !buf )
+    if( NULL == buf )
     {
         x264_log( h, X264_LOG_ERROR, "can't open file '%s'\n", filename );
         return -1;
Index: common/common.h
===================================================================
--- common/common.h	(.../x264-upstream)	(révision 18)
+++ common/common.h	(.../fixes1)	(révision 18)
@@ -78,14 +78,14 @@
 #endif
 
 #define CHECKED_MALLOC( var, size )\
-{\
+do {\
     var = x264_malloc( size );\
     if( !var )\
     {\
         x264_log( h, X264_LOG_ERROR, "malloc failed\n" );\
         goto fail;\
     }\
-}
+} while( 0 )
 
 #define X264_BFRAME_MAX 16
 #define X264_SLICE_MAX 4
@@ -263,6 +263,7 @@
     /* encoder parameters */
     x264_param_t    param;
 
+#define X264_THREAD_MAX X264_SLICE_MAX
     x264_t *thread[X264_SLICE_MAX];
 
     /* bitstream output */
@@ -324,7 +325,8 @@
         x264_frame_t *last_nonb;
 
         /* frames used for reference +1 for decoding + sentinels */
-        x264_frame_t *reference[16+2+1+2];
+#define X264_MAX_REFERENCES (16+2+1+2)
+        x264_frame_t *reference[X264_MAX_REFERENCES];
 
         int i_last_idr; /* Frame number of the last IDR */
 
Index: x264.h
===================================================================
--- x264.h	(.../x264-upstream)	(révision 18)
+++ x264.h	(.../fixes1)	(révision 18)
@@ -159,6 +159,7 @@
     int         b_bframe_adaptive;
     int         i_bframe_bias;
     int         b_bframe_pyramid;   /* Keep some B-frames as references */
+    int         i_direct_8x8_inference_flag; /* Use direct_8x8_inference_flag in SPS: (-1: 0 if possible according to level and partition mode) */
 
     int         b_deblocking_filter;
     int         i_deblocking_filter_alphac0;    /* [-6, 6] -6 light filter, 6 strong */
@@ -195,7 +196,8 @@
 
         int          i_me_method; /* motion estimation algorithm to use (X264_ME_*) */
         int          i_me_range; /* integer pixel motion estimation search range (from predicted mv) */
-        int          i_mv_range; /* maximum length of a mv (in pixels) */
+        int          i_horizontal_mv_range; /* maximum length of a horizontal mv (in frame pixels) */
+        int          i_vertical_mv_range; /* maximum length of a vertical mv (in frame pixels) */
         int          i_subpel_refine; /* subpixel motion estimation quality */
         int          b_bidir_me; /* jointly optimize both MVs in B-frames */
         int          b_chroma_me; /* chroma ME for subpel and mode decision in P-frames */
@@ -243,6 +245,16 @@
         char        *psz_zones;     /* alternate method of specifying zones */
     } rc;
 
+    /* b_allow_invalid_bitstreams allows to produce invalid bitstream by
+     * violating certains level contraints.
+     *
+     * This breaks interoperability so it should never be set except if you
+     * know in advance which decoder will decode the produced streams and
+     * you know that this decoder supports the constraints which are not
+     * respected.
+     * */
+    int b_allow_invalid_bitstreams;
+
     /* Muxing parameters */
     int b_aud;                  /* generate access unit delimiters */
     int b_repeat_headers;       /* put SPS/PPS before each keyframe */
